@@ -90,7 +90,7 @@ rquery_db_info <- function(...,
         if("schema" %in% names(qualifiers)) {
           dbi_id <- DBI::Id(schema = qualifiers$schema, table = as.character(id))
         } else {
-          dbi_id <- as.character(id) # sparklyr ‘0.8.4’ does not implement DBI::dbQuoteIdentifier for DBI::Id
+          dbi_id <- as.character(id) # sparklyr '0.8.4' does not implement DBI::dbQuoteIdentifier for DBI::Id
         }
         return(as.character(DBI::dbQuoteIdentifier(connection, dbi_id)))
       }
@@ -214,7 +214,7 @@ quote_table_name <- function(x, id,
     if("schema" %in% names(qualifiers)) {
       dbi_id <- DBI::Id(schema = qualifiers[["schema"]], table = as.character(id))
     } else {
-      dbi_id <- as.character(id) # sparklyr ‘0.8.4’ does not implement DBI::dbQuoteIdentifier for DBI::Id
+      dbi_id <- as.character(id) # sparklyr '0.8.4' does not implement DBI::dbQuoteIdentifier for DBI::Id
     }
     if(requireNamespace("DBI", quietly = TRUE)) {
       return(as.character(DBI::dbQuoteIdentifier(x, dbi_id)))
