@@ -145,6 +145,31 @@ to_sql.relop_rename_columns <- function (x,
                                          tnum = mk_tmp_name_source('tsql'),
                                          append_cr = TRUE,
                                          using = NULL) {
+  if(length(list(...))>0) {
+    stop("rquery::to_sql.relop_rename_columns unexpected arguments")
+  }
+  dispatch_to_sql_method(
+    method_name = "to_sql.relop_rename_columns",
+    x = x,
+    db = db,
+    limit = limit,
+    source_limit = source_limit,
+    indent_level = indent_level,
+    tnum = tnum,
+    append_cr = append_cr,
+    using = using)
+}
+
+to_sql_relop_rename_columns <- function(
+  x,
+  db,
+  ...,
+  limit = NULL,
+  source_limit = NULL,
+  indent_level = 0,
+  tnum = mk_tmp_name_source('tsql'),
+  append_cr = TRUE,
+  using = NULL) {
   wrapr::stop_if_dot_args(substitute(list(...)),
                           "rquery::to_sql.relop_rename_columns")
   qmap <- calc_used_relop_rename_columns(x, using=using)
